import "@picocss/pico/css/pico.min.css";
import "./style.css";
import { setupRequestPage } from "./routes/request.ts";
import { setupSendPage } from "./routes/send.ts";
import { setupReceivePage } from "./routes/receive.ts";
import { initKeyManager } from "./utils/keyManager.ts";

type PageState = {
  page: string;
  params: URLSearchParams;
};

let state: PageState = {
  page: "request",
  params: new URLSearchParams(window.location.hash.slice(1)),
};

const renderPage = () => {
  const appElement = document.querySelector<HTMLElement>("#app");

  if (!appElement) {
    throw new Error("No app element found");
  }

  switch (state.page) {
    case "request":
      setupRequestPage(appElement);
      break;
    case "send":
      setupSendPage(appElement, state.params.get("p")!);
      break;
    case "receive":
      setupReceivePage(appElement, {
        p: decodeURIComponent(state.params.get("p")!),
        iv: decodeURIComponent(state.params.get("iv")!),
        m: decodeURIComponent(state.params.get("m")!),
      });
      break;
  }
};

// Define a function to update the state and re-render
const setState = (newState: PageState) => {
  state = { ...state, ...newState };
  renderPage();
};

const update = () => {
  const params = new URLSearchParams(window.location.hash.slice(1));

  if (params.has("p") && params.has("iv") && params.has("m")) {
    setState({ page: "receive", params });
  } else if (params.has("p")) {
    setState({ page: "send", params });
  } else {
    setState({ page: "request", params });
  }
};

// Listen for changes in the hash and update the state accordingly
window.addEventListener("hashchange", update);

// Initialize the app
update();
renderPage();
initKeyManager(document.querySelector("#keyManagerContainer"));
