import { html, render } from "uhtml";
import { retrieveOrGenerateKeyPair } from "../utils/crypto.ts";

const URL_INPUT_ID = "request-url";
const COPY_BUTTON_ID = "copy-button";

export async function setupRequestPage(element: HTMLElement) {
  const url = await generateUrl();
  const template = html`
    <details open>
      <summary>How To Use:</summary>
      <ol>
        <li>Send the link below to someone you want to get a secret from.</li>
        <li>They add their secret and share the URL the app generates.</li>
        <li>Only your browser can open the URL and decrypt the secret.</li>
      </ol>
    </details>

    <form>
      <input
        type="text"
        name="request-url"
        .id=${URL_INPUT_ID}
        .value=${url.toString()}
        placeholder=""
        aria-label="Request URL"
        required
      />
      <button
        aria-label="Copy to Clipboard"
        .id=${COPY_BUTTON_ID}
        @click=${(event: Event) => copyToClipboard(event, url.toString())}
      >
        Copy to Clipboard
      </button>
    </form>
  `;

  render(element, template);
}

async function copyToClipboard(event: Event, url: string) {
  event.preventDefault();
  await navigator.clipboard.writeText(url);
  alert("Copied to clipboard!");
}

async function generateUrl(): Promise<URL> {
  const { publicKey: ecdhPublic } = await retrieveOrGenerateKeyPair();
  const ecdhPublicJwk = await window.crypto.subtle.exportKey("jwk", ecdhPublic);
  const url = new URL(window.location.toString());
  url.search = "";
  url.hash = `p=${btoa(JSON.stringify(ecdhPublicJwk))}`;
  return url;
}
